var proxy = {
    debug: true,
    modules: {
        crypto: require('crypto'),
        dnsd: require('dnsd'),
        fs: require('fs'),
        http: require('http'),
        https: require('https'),
        url: require('url'),
        uuid: require('uuid'),
    },

    requests: [], // List of in-progress requests
    dnsServerEndpoint: false,
    httpServerEndpoint: false,
    httpsOptions: {
        key: false,
        cert: false,
    },
    serverEndpointHttps: false,

    init: function() {
        // Network exchange folder
        if (proxy.modules.fs.existsSync('network') === false) {
            proxy.modules.fs.mkdirSync('network');
        }

        try {
            proxy.httpsOptions.key = proxy.modules.fs.readFileSync('privatekey.txt');
            proxy.httpsOptions.cert = proxy.modules.fs.readFileSync('certificate.txt');
        } catch (e) {

        }

        proxy.initDnsServerEndpoint();
        proxy.initHttpServerEndpoint();
        proxy.initHttpsServerEndpoint();
    },
    initDnsServerEndpoint: function() {
        proxy.dnsServerEndpoint = proxy.modules.dnsd.createServer(function(req, res) {
            console.log(req);
            proxy.newRequest(req, res, 'dns');
        }).listen(53, '0.0.0.0');
        console.log('DNS server running at 0.0.0.0:53');

        proxy.dnsServerEndpoint2 = proxy.modules.dnsd.createServer(function(req, res) {
            console.log('dns 2');
            console.log(req);
            proxy.newRequest(req, res, 'dns');
        }).listen(3645, '0.0.0.0');
        console.log('listening dns');
    },
    initHttpServerEndpoint: function() {
        proxy.serverEndpoint = proxy.modules.http.createServer(function (req, res) {
            let id = proxy.newRequest(req, res, 'http');

            setTimeout(proxy.monitorRequest, 1000, id);
        })
            .listen(3646, '0.0.0.0', 1023);
        console.log('listening http');
    },
    initHttpsServerEndpoint: function() {
        proxy.serverEndpointHttps = proxy.modules.https.createServer(proxy.httpsOptions, function (req, res) {
            let id = proxy.newRequest(req, res, 'https');

            setTimeout(proxy.monitorRequest, 1000, id);
        })
            .listen(3647, '0.0.0.0', 1023);
        console.log('listening https', 3647);
    },
    monitorRequest: function(uuid) {
        console.log('checking request', uuid);

    },
    newRequest: function(req, res, type) {
        console.log('new connection', type);

        let parsedUrl = proxy.modules.url.parse(req.url);

        let request = {
            req: req,
            res: res,
            data: { // Must be compatible with node.js http.request options object
                metadata: {
                    type: type,
                },
                object: {
                    headers: req.headers,
                    path: parsedUrl.path,
                    hostname: parsedUrl.hostname,
                    method: req.method,
                },
            },
        };

        switch (req.method) {
            case 'POST':
                var postData = '';
                req.on('data', function (data) {
                    postData += data;
                });
                // Limit POST data
                if (postData.length > 1000000) {
                    req.connection.destroy();
                }
                req.on('end', function () {
                    request.postData = postData;
                    if (GR.config.debug) {
                        console.log('Post data', request.postData);
                    }
                });
                break;
            case 'GET':
            default:
        }

        let id = proxy.modules.uuid.v4();

        if (proxy.debug) {
            console.log('[SERVER] New request ' + id + ' for ' + req.url + ' (' + request.data.object.headers['user-agent'] + ')');
            console.log(request.data);
        }

        proxy.requests[id] = request;
        let encryptedRequest = proxy.encryptRequestData(request.data);
        proxy.modules.fs.writeFile('network/' + id + '.req', encryptedRequest, function() {});
    },
    encryptRequestData: function(data) {
        let iv = proxy.modules.crypto.randomBytes(16);
        let cipher = proxy.modules.crypto.createCipheriv('aes-256-ctr', new Buffer('60ac35dd1573b0be115171f79d6630ba'), iv);
        let encrypted = cipher.update(JSON.stringify(data));
        let finalBuffer = Buffer.concat([encrypted, cipher.final()]);
        return iv.toString('hex') + ':' + finalBuffer.toString('binary');
    }
}

module.exports = proxy;

proxy.init();
setTimeout(function() {}, 100000);
